      SUBROUTINE DHRDT1( N, ILO, IHI, A, LDA, AF, LDAF, Q, LDQ, H, LDH,
     $                   WORK, LWORK, RWORK, RESULT )
*
*  -- LAPACK test routine --
*     E. Anderson, Cray Research Inc.
*     May 25, 1995
*     12-1-99:  Change calls to DORT01 to DORGT1  (eca)
*
*     .. Scalar Arguments ..
      INTEGER            IHI, ILO, LDA, LDAF, LDH, LDQ, LWORK, N
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION   A( LDA, * ), AF( LDAF, * ), H( LDH, * ),
     $                   Q( LDQ, * ), RESULT( * ), RWORK( * ),
     $                   WORK( LWORK )
*     ..
*
*  Purpose
*  =======
*
*  DHRDT1 tests the reduction to Hessenberg form of an n-by-n matrix A
*  as computed by SGEHRD, and the generation of the orthogonal matrix
*  from the reduction as computed by DORGHR.
*
*  Given A, its factored form AF, and the orthogonal matrix Q, DHRDT1
*  compares the upper Hessenberg part of A with Q'*A*Q, and checks that
*  the columns of Q are orthonormal.
*
*  Arguments
*  =========
*
*  N       (input) INTEGER
*          The number of rows and columns of the original matrix A.
*          N >= 0.
*
*  ILO     (input) INTEGER
*  IHI     (input) INTEGER
*          It is assumed that A is already upper triangular in rows
*          and columns 1:ILO-1 and IHI+1:N. ILO and IHI are normally
*          set by a previous call to SGEBAL; otherwise they should be
*          set to 1 and N respectively.  See SGEHRD for further details.
*          1 <= ILO <= IHI <= N, if N > 0; ILO=1 and IHI=0, if N=0.
*
*  A       (input) DOUBLE PRECISION array, dimension (LDA,N)
*          The initial n-by-n matrix A.
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.
*
*  AF      (input) DOUBLE PRECISION array, dimension (LDAF,N)
*          Details of the reduction to Hessenberg form of the matrix A,
*          as returned by SGEHRD.
*
*  LDAF    (input) INTEGER
*          The leading dimension of the array AF.
*
*  Q       (input) DOUBLE PRECISION array, dimension (LDQ,N)
*          The n-by-n orthogonal matrix Q, as returned by DORGHR.
*
*  LDQ     (input) INTEGER
*          The leading dimension of the array Q.
*
*  H       (workspace) DOUBLE PRECISION array, dimension (LDH,N)
*
*  LDH     (input) INTEGER
*          The leading dimension of the array H.
*
*  WORK    (workspace) DOUBLE PRECISION array, dimension (LWORK)
*
*  LWORK   (input) INTEGER
*          The length of WORK.  Assumed to be at least N*N.
*
*  RWORK   (workspace) DOUBLE PRECISION array, dimension (N)
*
*  RESULT  (output) DOUBLE PRECISION array, dimension (2)
*          The test ratios:
*          RESULT(1) = norm( H - Q'*A*Q ) / ( N * norm(A) * EPS )
*          RESULT(2) = norm( I - Q'*Q ) / ( N * EPS )
*
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION   ZERO, ONE
      PARAMETER          ( ZERO = 0.0D+0, ONE = 1.0D+0 )
*     ..
*     .. Local Scalars ..
      INTEGER            LDWORK
      DOUBLE PRECISION   ANORM, EPS, RESID
*     ..
*     .. External Functions ..
      DOUBLE PRECISION   DLAMCH, DLANGE
      EXTERNAL           DLAMCH, DLANGE
*     ..
*     .. External Subroutines ..
      EXTERNAL           DCOPY, DGEMM, DLACPY, DLASET, DORGT1
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, DBLE
*     ..
*     .. Executable Statements ..
*
*     Copy the Hessenberg matrix to H.
*
      CALL DLACPY( 'Upper', N, N, AF, LDAF, H, LDH )
      CALL DLASET( 'Lower', N-1, N, ZERO, ZERO, H( 2, 1 ), LDH )
      IF( ILO.LT.IHI )
     $   CALL DCOPY( IHI-ILO, AF( ILO+1, ILO ), LDAF+1, H( ILO+1, ILO ),
     $               LDH+1 )
*
*     Compute Q' * A
*
      LDWORK = MAX( 1, N )
      CALL DGEMM( 'Transpose', 'No transpose', N, N, N, ONE, Q, LDQ, A,
     $            LDA, ZERO, WORK, LDWORK )
*
*     Compute H - (Q' * A) * Q
*
      CALL DGEMM( 'No transpose', 'No transpose', N, N, N, -ONE, WORK,
     $            LDWORK, Q, LDQ, ONE, H, LDH )
*
*     Compute norm( H - Q'*A*Q ) / ( N * norm(A) * EPS ) .
*
      EPS = DLAMCH( 'Epsilon' )
      ANORM = DLANGE( '1', N, N, A, LDA, RWORK )
      RESID = DLANGE( '1', N, N, H, LDH, RWORK )
      IF( ANORM.GT.ZERO ) THEN
         RESULT( 1 ) = ( ( RESID / DBLE( MAX( 1, N ) ) ) / ANORM ) / EPS
      ELSE
         RESULT( 1 ) = ZERO
      END IF
*
*     Compute norm( I - Q'*Q ) / ( N * EPS ) .
*
      CALL DORGT1( 'Columns', N, N, Q, LDQ, WORK, LWORK, RWORK,
     $             RESULT( 2 ) )
      RETURN
*
*     End of DHRDT1
*
      END
